/*
 * Simulator of microcontrollers (timer0.cc)
 *
 * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of microcontroller simulator: ucsim.

UCSIM is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UCSIM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UCSIM; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "timer0cl.h"
#include "regs51.h"


cl_timer0::cl_timer0(class cl_uc *auc, int aid, char *aid_string):
  cl_hw(auc, HW_TIMER, aid, aid_string)
{
  uc51= (class t_uc51 *)auc;
  cell_tmod= cell_tcon= 0;
  if (aid == 0)
    {
      mask_M0  = bmM00;
      mask_M1  = bmM10;
      mask_C_T = bmC_T0;
      mask_GATE= bmGATE0;
      mask_TR  = bmTR0;
      mask_INT = bm_INT0;
    }
  else if (aid == 1)
    {
      mask_M0  = bmM01;
      mask_M1  = bmM11;
      mask_C_T = bmC_T1;
      mask_GATE= bmGATE1;
      mask_TR  = bmTR1;
      mask_INT = bm_INT1;
    }
  else
    {
    }
}

int
cl_timer0::init(void)
{
  class cl_mem *sfr= uc->mem(MEM_SFR);

  if (sfr)
    {
      cell_tmod= sfr->register_hw(TMOD, this, 0);
      cell_tcon= sfr->register_hw(TCON, this, 0);
    }
  return(0);
}

/*t_mem
cl_timer0::read(class cl_cell *cell)
{
  return(cell->get());
}*/

void
cl_timer0::write(class cl_cell *cell, t_mem *val)
{
  if (cell == cell_tmod)
    {
      t_mem md= *val & (mask_M0|mask_M1);
      if (md == mask_M0)
	mode= 1;
      else if (md == mask_M1)
	mode= 2;
      else if (md == (mask_M0|mask_M1))
	mode= 3;
      else
	mode= 0;
      GATE= *val & mask_GATE;
      C_T = *val & mask_C_T;
    }
  else if (cell == cell_tcon)
    {
      TR= *val & mask_TR;
    }
}

void
cl_timer0::mem_cell_changed(class cl_mem *mem, t_addr addr)
{
  class cl_mem *s= uc->mem(MEM_SFR);
  class cl_cell *c= 0;

  if (mem && s && mem == s)
    {
      switch (addr)
	{
	case TMOD:
	  c= cell_tmod= s->get_cell(TMOD);
	  break;
	case TCON:
	  c= cell_tcon= s->get_cell(TCON);
	  break;
	}
      if (c)
	{
	  t_mem d= c->get();
	  write(c, &d);
	}
    }
}

int
cl_timer0::tick(int cycles)
{
  uint tmod= uc51->sfr->get(TMOD);
  uint tcon= uc51->sfr->get(TCON);
  uint p3= uc51->sfr->get(P3);

  if (!TR)
    return(0);
  INT= p3 & uc51->port_pins[3] & mask_INT;
  if (GATE && !INT)
    return(0);

  switch (mode)
    {
    case 0: do_mode0(cycles); break;
    case 1: do_mode1(cycles); break;
    case 2: do_mode2(cycles); break;
    case 3: do_mode3(cycles); break;
    }
  //return(0);

  if (((tmod & bmGATE0) &&
       (p3 & uc51->port_pins[3] & bm_INT0)) ||
      (tcon & bmTR0))
    {
      if (!(tmod & bmC_T0) ||
	  ((uc51->prev_p3 & bmT0) &&
	   !(p3 & uc51->port_pins[3] & bmT0)))
	{
	  if (!(tmod & bmM00) &&
	      !(tmod & bmM10))
	    {
	      if (tmod & bmC_T0)
		cycles= 1;
	      while (cycles--)
		{
		  // mod 0, TH= 8 bit t/c, TL= 5 bit precounter
		  uc51->sfr->add(TL0, 1);
		  if ((uc51->sfr->get(TL0) & 0x1f) == 0)
		    {
		      uc51->sfr->set(TL0, 0);
		      if (!uc51->sfr->add(TH0, 1))
			{
			  uc51->sfr->set_bit1(TCON, bmTF0);
			  uc51->t0_overflow();
			}
		    }
		}
	    }
	  else if ((tmod & bmM00) &&
		   !(tmod & bmM10))
	    {
	      if (tmod & bmC_T0)
		cycles= 1;
	      while (cycles--)
		{
		  // mod 1 TH+TL= 16 bit t/c
		  if (!uc51->sfr->add(TL0, 1))
		    {
		      if (!uc51->sfr->add(TH0, 1))
			{
			  uc51->sfr->set_bit1(TCON, bmTF0);
			  uc51->t0_overflow();
			}
		    }
		}
	    }
	  else if (!(tmod & bmM00) &&
		   (tmod & bmM10))
	    {
	      if (tmod & bmC_T0)
		cycles= 1;
	      while (cycles--)
		{
		  // mod 2 TL= 8 bit t/c auto reload from TH
		  if (!uc51->sfr->add(TL0, 1))
		    {
		      uc51->sfr->set(TL0, uc51->sfr->get(TH0));
		      uc51->sfr->set_bit1(TCON, bmTF0);
		      uc51->t0_overflow();
		    }
		}
	    }
	  else
	    {
	      // mod 3 TL= 8 bit t/c
	      //       TH= 8 bit timer controlled with T1's bits
	      if (!uc51->sfr->add(TL0, 1))
		{
		  uc51->sfr->set_bit1(TCON, bmTF0);
		  uc51->t0_overflow();
		}
	    }
	}
    }
  if ((tmod & bmM00) &&
      (tmod & bmM10))
    {
      if (((tmod & bmGATE1) &&
	   (p3 & uc51->port_pins[3] & bm_INT1)) ||
	  (tcon & bmTR1))
	{
	  if (!uc51->sfr->add(TH0, 1))
	    {
	      uc51->sfr->set_bit1(TCON, bmTF1);
	      uc51->s_tr_t1++;
	      uc51->s_rec_t1++;
	      uc51->t0_overflow();
	    }
	}
    }
  return(resGO);
}

int
cl_timer0::do_mode0(int cycles)
{
  return(0);
}

int
cl_timer0::do_mode1(int cycles)
{
  return(0);
}

int
cl_timer0::do_mode2(int cycles)
{
  return(0);
}

int
cl_timer0::do_mode3(int cycles)
{
  return(0);
}

void
cl_timer0::print_info(class cl_console *con)
{
  char *modes[]= { "13 bit", "16 bit", "8 bit autoreload", "2x8 bit" };
  int tmod= uc->get_mem(MEM_SFR, TMOD);
  int on;

  con->printf("%s[%d] 0x%04x", id_string, id,
	      256*uc->get_mem(MEM_SFR, TH0)+uc->get_mem(MEM_SFR, TL0));
  int mode= tmod & (bmM00|bmM10);
  con->printf(" %s", modes[mode]);
  con->printf(" %s", (tmod&bmC_T0)?"counter":"timer");
  if (tmod&bmGATE0)
    {
      con->printf(" gated");
      on= uc->get_mem(MEM_SFR, P3) & uc->port_pins[3] & bm_INT0;
    }
  else
    on= uc->get_mem(MEM_SFR, TCON) & bmTR0;
  con->printf(" %s", on?"ON":"OFF");
  con->printf(" irq=%c", (uc->get_mem(MEM_SFR, TCON)&bmTF0)?'1':'0');
  con->printf(" %s", (uc->get_mem(MEM_SFR, IE)&bmET0)?"en":"dis");
  con->printf(" prio=%d", uc->it_priority(bmPT0));
  con->printf("\n");
}


/* End of s51.src/timer0.cc */
